////////////////////////////////////////////////////////////////////////////
//
//  parser.h
//
//	Simple Parser Header File
//
//	Author 	: Rene Bllhoff
//  Created : around Jan. 2005
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////


  #ifndef _SIMPLE_PARSER_H_
    #define _SIMPLE_PARSER_H_  
    
    // define parser configuration file
    
    #define PARSER_CONFIG_FILE "parser_cfg.h"

    // parser types
  
    // function pointer
    
    // callback for user defined parser codecs
    typedef  UCHAR (* T_PARSE_FUNC) (UCHAR ucOpcode, UCHAR ucChar, UCHAR ucLastToken);    
		//
		//	ucOpcode			= Opcode from Parser to Codec
		//									- EC_RESET    - Reset Internal State of codec
    //									- EC_PROCESS  - Process current Char
    //  ucChar				= Current Char
		//  ucLastToken		= Token which lead to call this function 
		//  return value  = State of Processing
		//                   TK_SEARCHING - decoding still in progress
		//									 TK_NONE      - error in text -> exit
		//									 TK_UNGET     - finish parsing, drop token and process current char 
		//									 else         - finish parsing
		//

    // callback for executing found 
    typedef  VOID  (* T_PARSE_EXEC) (UCHAR ucParse, UCHAR *pucParseData);
		//
		//	ucParse				= Parse Number
		//	pucParseData	= Pointer to Value Buffer
		//
    
    // tools for T_PARSE_EXEC for accessing value buffer (assuming : Parameter-name for UCHAR * in defined function is named "pucParseData")
		// byte order in value array is big endian
    #define PARSE_DATA_UCHAR(x)  ( ((UCHAR) (pucParseData [x + 0])) )
    #define PARSE_DATA_UINT(x)   ( ((UINT)  (pucParseData [x + 0]) <<  8) | ((UINT)  (pucParseData [x + 1]) <<  0) )
    #define PARSE_DATA_ULONG(x)  ( ((ULONG) (pucParseData [x + 0]) << 24) | ((ULONG) (pucParseData [x + 1]) << 16) | ((ULONG) (pucParseData [x + 2]) << 8) | ((UINT)  (pucParseData [x + 3]) << 0) )
  
    // type for definition of tokens (keywords, symbols & ctrl chars (0x0d))
    typedef struct {
      UCHAR ucTokenAttribute;		// attribute or codec
      UCHAR ucLength;						// length of text 
      UCHAR *pucTokenText;		  // pointer to text
    } TOKEN_DEF;
  
    // type for callback function for found parse
    typedef struct {
      UCHAR         ucNumberTokens;		// number of tokens in this parse-line
      UCHAR				 *pstParseElements;	// pointer to tokens of parser-line
      T_PARSE_EXEC  pfCallFunction;		// if successfull, call this function
    } PARSE_LINE;    


    // load parser-configuration
    
    #define TOKEN_CONFIG
      #include PARSER_CONFIG_FILE
    #undef TOKEN_CONFIG
    

    // define enums (token control/codecs, tokens, parse-lines)
  
		// enums for parser-codecs
    #define TOKEN_CONTROL(kind,handleproc) kind,
      enum eTOKEN_CONTROL {
				TC_END,			// exeucte attribute
				TC_STD,			// standard attribute (normal token)
				TC_FOLLOW,	// tokens like space (multiple treated as one)
        #include PARSER_CONFIG_FILE
        NUM_TOKEN_CONTROLS
      };
    #undef TOKEN_CONTROL
  
		// enums for parser tokens
    #define TOKEN(token,control,word) token,
      enum eTOKEN_LIST {
        #include PARSER_CONFIG_FILE
        NUM_TOKENS,
        TK_SEARCHING = 0xfd,
        TK_UNGET,
        TK_NONE        
      };        
    #undef TOKEN
      
		// enums for parser-lines
		#define PARSE_LINES
			#define PARSE_BEGIN(parse, func) parse,
			#define PARSE_DEF(value)  
			#define PARSE_END      
				enum ePARSES_LIST {
					#include PARSER_CONFIG_FILE
					NUM_PARSES,
					NO_PARSE = 0xff
				};        
			#undef PARSE_BEGIN
			#undef PARSE_DEF  
			#undef PARSE_END  
		#undef PARSE_LINES


    // define external parser codec functions
    
    #define TOKEN_EXTERNALS(func) extern UCHAR func (UCHAR, UCHAR, UCHAR);
      #include PARSER_CONFIG_FILE
    #undef TOKEN_EXTERNALS    
    
		// define external callback functions for found parses
		#define PARSE_LINES
			#define PARSE_BEGIN(parse, func) extern VOID func (UCHAR, UCHAR *);
			#define PARSE_DEF(value)  
			#define PARSE_END      
				#include PARSER_CONFIG_FILE
			#undef PARSE_BEGIN
			#undef PARSE_DEF  
			#undef PARSE_END       
    #undef PARSE_LINES
    
    
    //  external parse process defines
    
    enum TK_EXT_OPCODES {
      EC_RESET,
      EC_PROCESS
    };
    
		// useful functions

		#define ucUpperCase(x)           (  ((x >= 'a') && (x <= 'z')) ? (x ^ 0x20) : (x) ) 

    
		// Initialize all Data
    extern VOID   vParserInit						(VOID);
		// Drop Byte to Value-Array (interpretation has to be done seperately)
    extern VOID   vParserDropValue			(UCHAR ucValue);
		// Processes characters and strings
    extern VOID   vParserProcessChar		(UCHAR ucChar);
    extern VOID   vParserProcessString  (UCHAR *pucChar);


    #endif
