////////////////////////////////////////////////////////////////////////////
//
//  parser_ext.c
//
//	2 Parser Codecs (Hexvalue (32-bit) & String Codec (null terminated)
//
//	Author 	: Rene Bllhoff
//  Created : around Jan. 2005
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////

  #include "lib.h"


// useful macros 

#define IS_NIBBLE(x)        ( (( ucUpperCase (x) >= 'A' ) && ( ucUpperCase (x) <= 'F')) || (( x >= '0' ) && ( x <= '9')) )
#define ASCII_TO_NIBBLE(x)  ( ( ucUpperCase (x) >= 'A' ) ?  ( ucUpperCase (x) - 'A' + 10 ) : ( (x) - '0') )

// global codec variables

static ULONG  ulValue;
static UCHAR  ucHexCount;

// hexadecimal codec (introduced by 0x)
// byte order in value array is big endian

UCHAR ucHexValueParse (UCHAR ucOpcode, UCHAR ucChar, UCHAR ucLastToken)
{
  switch (ucOpcode)
  {
		// reset global codec data for new input
    case EC_RESET  :
      ulValue = 0;
      ucHexCount = 0;
      break;
		// process new character (and drop 32-bit value if needed) 
    case EC_PROCESS  :
      if (IS_NIBBLE (ucChar)) // only when character is hex-char ..
      {
        ulValue <<= 4;
        ulValue |= ( ASCII_TO_NIBBLE (ucChar) & 0x0f );
        ucHexCount ++;
        if (ucHexCount == 8)   // drop value when 8 digits are entered without error
        {
          vParserDropValue ( (UCHAR) ((ulValue >> 24) & 0xff) );	// big endian (might be changed)
          vParserDropValue ( (UCHAR) ((ulValue >> 16) & 0xff) );        
          vParserDropValue ( (UCHAR) ((ulValue >>  8) & 0xff) );
          vParserDropValue ( (UCHAR) ((ulValue >>  0) & 0xff) );
          return TK_HEXVALUE;  // exit codec
        }
        return TK_SEARCHING;	// still possible characters to process
      }
      else		// otherwise end processing, drop value ...
      {
        vParserDropValue ( (UCHAR) ((ulValue >> 24) & 0xff) );	// big endian (might be changed)
        vParserDropValue ( (UCHAR) ((ulValue >> 16) & 0xff) );
        vParserDropValue ( (UCHAR) ((ulValue >>  8) & 0xff) );
        vParserDropValue ( (UCHAR) ((ulValue >>  0) & 0xff) );
        return TK_UNGET;  // ... and exit hex-codec and (re-)process current char (we want to "unget"). it might be a space :-)
      }
  }
  return TK_NONE; // exit codec (undefined state)
}


// string codec (introduced by ")

UCHAR ucStringValueParse (UCHAR ucOpcode, UCHAR ucChar, UCHAR ucLastToken)
{
  switch (ucOpcode)
  {
		// nothing to reset
    case EC_RESET  :
      break;
		// process new character. simply drop current character into value array ...
    case EC_PROCESS  :
      if (ucChar != '"')	// ... if character's not a "
      {
        vParserDropValue (ucChar);
        return TK_SEARCHING;				// keep on waiting for "
      }
      else
      {
        vParserDropValue (0x00);		// when current char is " then drop a 0 for terminating string
        return ucLastToken;					// and finish string codec
      }
  }
  return TK_NONE; // exit codec (undefined state)
}

