////////////////////////////////////////////////////////////////////////////
//
//  parser_cfg.h
//
//	Example of configuring the Parser
//    - implements following commands
//      - "LED ON"  
//      - "LED OFF"
//      - "READ 0xXXXXXXXX"
//      - "WRITE 0xXXXXXXXX 0xYYYYYYYY"
//
//
//	Author 	: Rene Bllhoff
//  Created : around Jan. 2005
//
//	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////



  // parser buffer sizes

  #ifdef TOKEN_CONFIG
  
		// defines maximum number of tokens for a parse-line
    #define SIZE_TOKEN_BUFFER			16   
    // defines maximum size of value buffer in bytes
    #define SIZE_VALUE_BUFFER			24
		// global character to reset parser 
    #define TOKEN_RESET_CHAR			0x1b 
  #endif


	// define characters to be ignored (single, range) 

  #ifdef TOKEN_IGNORE_CHARS
    CHAR_IGNORE_AREA0  (      12)    // (0 - 12)  gap for cr (0x0d) 
		CHAR_IGNORE_AREA   ( 14,  31)    // (14 - 31) note : esc must not have a gap, its "globally" defined
    CHAR_IGNORE_AREA   (128, 255)    // all characters equal or above 128
	#endif
  
	// define external parser codec functions  

  #ifdef TOKEN_EXTERNALS
    TOKEN_EXTERNALS (ucHexValueParse)			// codec for 0xXXXX values (always 32-bit long)
    TOKEN_EXTERNALS (ucStringValueParse)  // codec for string values (ends with 0)  
  #endif  

  // defines parser codec functions  

  #ifdef TOKEN_CONTROL
    TOKEN_CONTROL (TC_VALUE   , ucHexValueParse)		// token attribute for hex-codec
    TOKEN_CONTROL (TC_STRING  , ucStringValueParse)	// token attribute for string 
  #endif

	// define token

  #ifdef TOKEN

	// Template :	
	//TOKEN (name,							attribute, text)

		// execute token(s)

    TOKEN (TK_EOL,            TC_END,    "\x0d")

		// useful tokens
		
		TOKEN (TK_EQUAL,          TC_STD,    "=")
    TOKEN (TK_DOT,            TC_STD,    ".")
    TOKEN (TK_SEP,            TC_STD,    ",")
    TOKEN (TK_SPACE,          TC_FOLLOW, " ")
    TOKEN (TK_QUERY,          TC_STD,    "?")

		// codec tokens

    TOKEN (TK_HEXVALUE,       TC_VALUE,  "0x")			// 0x calls hex codec
    TOKEN (TK_STRINGVALUE,    TC_STRING, "\x22")		// "  calls string codec

    //
    // command tokens
    //

    TOKEN (TK_LED,            TC_STD,    "LED")
    TOKEN (TK_ON,             TC_STD,    "ON")
    TOKEN (TK_OFF,            TC_STD,    "OFF")
    TOKEN (TK_READ,           TC_STD,    "READ")
    TOKEN (TK_WRITE,          TC_STD,    "WRITE")
    TOKEN (TK_IO,             TC_STD,    "IO")

  #endif
  
	// define parse lines 

  
  #ifdef PARSE_LINES

	// Template :	
	//PARSE_BEGIN (name         , function)
	//  PARSE_DEF   (expectedtoken)
	//  PARSE_DEF   (...)
	//  ...
	//PARSE_END


		// parse-line for led on in function vSetLED
    PARSE_BEGIN (PARSER_LED_ON, vSetLED)        // "LED ON"
      PARSE_DEF   (TK_LED)      // "LED"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_ON)       // "ON"
    PARSE_END    

		// parse-line for led off in function vSetLED
    PARSE_BEGIN (PARSER_LED_OFF, vSetLED)       // "LED OFF"
      PARSE_DEF   (TK_LED)      // "LED"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_OFF)      // "OFF"
    PARSE_END    
    
		// parse-line for read io with address in function vIOFuncs (32-bit)
    PARSE_BEGIN (PARSER_READ_IO, vIOFuncs)      // "READ IO 0x..."
      PARSE_DEF   (TK_READ)     // "READ"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_IO)       // "IO"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_HEXVALUE) // "0xyyyyyyyy"
    PARSE_END     

		// parse-line for write io with address in function vIOFuncs (32-bit)
    PARSE_BEGIN (PARSER_WRITE_IO, vIOFuncs)      // "WRITE IO 0x... 0x..."
      PARSE_DEF   (TK_WRITE)    // "WRITE"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_IO)       // "IO"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_HEXVALUE) // "0xyyyyyyyy"
      PARSE_DEF   (TK_SPACE)    // " "
      PARSE_DEF   (TK_HEXVALUE) // "0xyyyyyyyy"      
    PARSE_END    

  #endif
